import { Application, Request, Response } from 'express'
import UserService from '../services/userService'
import jwt from 'jsonwebtoken'
import config from './config'
import { loginAccountSchema, registerAccountSchema } from './zodTypes'
import { TAsset } from './types'
import tileRepository from '../repositories/tileRepository'
import objectRepository from '../repositories/objectRepository'
import spriteRepository from '../repositories/spriteRepository'
import fs from 'fs'
import { httpLogger } from './logger'
import { getPublicPath } from './storage'

async function addHttpRoutes(app: Application) {
  /**
   * Get all base sprite, assets
   * @param req
   * @param res
   */
  app.get('/assets', async (req: Request, res: Response) => {
    let assets: TAsset[] = []

    const sprites = await spriteRepository.getAll()
    const tiles = await tileRepository.getAll()
    const objects = await objectRepository.getAll()

    // sprites all contain spriteActions, loop through these
    sprites.forEach((sprite) => {
      sprite.spriteActions.forEach((spriteAction) => {
        assets.push({
          key: sprite.id + '-' + spriteAction.action,
          url: '/assets/sprites/' + sprite.id + '/' + spriteAction.action + '.png',
          group: spriteAction.isAnimated ? 'sprite_animations' : 'sprites',
          updatedAt: sprite.updatedAt,
          frameCount: JSON.parse(JSON.stringify(spriteAction.sprites)).length,
          frameWidth: spriteAction.frameWidth,
          frameHeight: spriteAction.frameHeight,
        })
      })
    })

    // Get all tiles
    tiles.forEach((tile) => {
      assets.push({
        key: tile.id,
        url: '/assets/tiles/' + tile.id + '.png',
        group: 'tiles',
        updatedAt: tile.updatedAt
      })
    })

    // Get all objects
    objects.forEach((object) => {
      assets.push({
        key: object.id,
        url: '/assets/objects/' + object.id + '.png',
        group: 'objects',
        updatedAt: object.updatedAt
      })
    })

    res.json(assets)
  })

  /**
   * Get a specific asset
   * @param req
   * @param res
   */
  app.get('/assets/:type/:spriteId?/:file', (req: Request, res: Response) => {
    const assetType = req.params.type
    const spriteId = req.params.spriteId
    const fileName = req.params.file

    let assetPath
    if (assetType === 'sprites' && spriteId) {
      assetPath = getPublicPath(assetType, spriteId, fileName)
    } else {
      assetPath = getPublicPath(assetType, fileName)
    }

    if (!fs.existsSync(assetPath)) {
      httpLogger.error(`File not found: ${assetPath}`)
      return res.status(404).send('Asset not found')
    }

    res.sendFile(assetPath, (err) => {
      if (err) {
        httpLogger.error('Error sending file:', err)
        res.status(500).send('Error downloading the asset')
      }
    })
  })

  /**
   * Login
   * @param req
   * @param res
   */
  app.post('/login', async (req: Request, res: Response) => {
    const { username, password } = req.body

    try {
      loginAccountSchema.parse({ username, password })
    } catch (error: any) {
      return res.status(400).json({ message: error.errors[0]?.message })
    }

    const userService = new UserService()
    const user = await userService.login(username, password)

    if (user && typeof user !== 'boolean') {
      const token = jwt.sign({ id: user.id }, config.JWT_SECRET, { expiresIn: '4h' })
      return res.status(200).json({ token })
    }

    return res.status(400).json({ message: 'Failed to login' })
  })

  /**
   * Register
   * @param req
   * @param res
   */
  app.post('/register', async (req: Request, res: Response) => {
    const { username, password } = req.body

    try {
      registerAccountSchema.parse({ username, password })
    } catch (error: any) {
      return res.status(400).json({ message: error.errors[0]?.message })
    }

    const userService = new UserService()
    const user = await userService.register(username, password)

    if (user) {
      return res.status(200).json({ message: 'User registered' })
    }

    return res.status(400).json({ message: 'Failed to register user' })
  })

  httpLogger.info('Web routes added')
}

export { addHttpRoutes }
